package org.gcube.portlets.user.geoportaldataviewer.client;

import org.gcube.portlets.user.geoportaldataviewer.client.gis.OsmExample;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.GeonaDataViewMainPanel;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class GeoportalDataViewer implements EntryPoint {
  /**
   * The message displayed to the user when the server cannot be reached or
   * returns an error.
   */
  private static final String SERVER_ERROR = "An error occurred while "
      + "attempting to contact the server. Please check your network "
      + "connection and try again.";

  /**
   * Create a remote service proxy to talk to the server-side Greeting service.
   */
  private final GeoportalDataViewerServiceAsync greetingService = GWT.create(GeoportalDataViewerService.class);
  
  public final static String APP_DIV = "geoportal-data-viewer";

  private boolean initialized = false;
  
  private OsmExample osmExample = null;
  
  private GeonaDataViewMainPanel mainPanel;
  /**
   * This is the entry point method.
   */
  public void onModuleLoad() {
	  
	mainPanel = new GeonaDataViewMainPanel();
	
	RootPanel.get(APP_DIV).add(mainPanel);
	
	Scheduler.get().scheduleDeferred(new ScheduledCommand() {
		
		@Override
		public void execute() {
			osmExample = new OsmExample(mainPanel.getMapPanel().getElement().getId());
			mainPanel.setMap(osmExample);
			
		}
	});
	
	Window.addResizeHandler(new ResizeHandler() {
		@Override
		public void onResize(ResizeEvent event) {
			GWT.log("onWindowResized width: "+event.getWidth()+" height: "+event.getHeight());
			mainPanel.setHeight(event.getHeight()-200);
		}
	});
	
	greetingService.greetServer("SERVICE IMPL", new AsyncCallback<String>() {
		
		@Override
		public void onSuccess(String result) {
			
		}
		
		@Override
		public void onFailure(Throwable caught) {
			
			
		}
	});
  }
  

}
